<?php

/*
	Creates a colorized version of a PNG template image (usually a greyscale image) based
	on a hexadecimal color.
	
	Usage: colorize_png.php?color=dadcff&template=tab_bg.png
	
	Last Modified:	2007-01-09 by R. P. Vullo
	Last Modified:	2007-01-11 by R. P. Vullo
	Last Modified:	2009-5-27 by R. P. Vullo
	Last Modified:	2012-1-19 by R. P. Vullo
	Last Modified:	2013-10-7 by R. P. Vullo
		Added headers to tell the browser to cache the generated image. Be sure to
		modify while testing! (Lines 99-101)
	Last Modified:	2014-12-20 by R. P. Vullo
		added ability to specify output image type via mime parameter and molly.ini
		(in response to Mac OS X Yosemite dropping support for png in GD)
*/


$gv_Color = m_Hex2int(m_ValidHexColor($_REQUEST['color'],'888888'));
$gv_Template = m_SafeString($_REQUEST['template']);

if (isset($_REQUEST['mime'])) {
	$gv_MIME = $_REQUEST['mime'];
} else {
	if (isset($gv_SiteGlobals['Defaults']['ImageMIME'])) {
		$gv_MIME = $gv_SiteGlobals['Defaults']['ImageMIME'];
	} else {
		$gv_MIME = 'image/png';
	}
}

m_colorize ($gv_Template, $gv_Color['red'], $gv_Color['green'], $gv_Color['blue']);


function m_colorize ($pv_Template, $pv_Red, $pv_Green, $pv_Blue) {
	global $gv_MIME;
	
/*	Parameters:	$pv_Template - a valid path to a (greyscale) PNG-8 image (indexed, with a palette)
				$pv_Red, $pv_Green, $pv_Blue to be integers in the range 0 - 255
	Result: outputs a png image colorized with the color represented by $pv_Red, $pv_Green, $pv_Blue.
*/
	

//	Load the Template

	$fv_TemplatePathInfo = pathinfo($pv_Template);
	$gv_TemplateFileExtension = strtolower($fv_TemplatePathInfo['extension']);

	switch ($gv_TemplateFileExtension) {
		case 'png':
			$fv_TemplateImage = @imagecreatefrompng($pv_Template);
		break;
		
		case 'jpg':
		case 'jpeg':
			$fv_TemplateImage = @imagecreatefromjpeg($pv_Template);
		break;
		
		case 'gif':
			$fv_TemplateImage = @imagecreatefromgif($pv_Template);
		break;
	}

		if (!$fv_TemplateImage) { //oops, an error - let's attempt to debug
		
		   /* Create a blank image */
			$im  = imagecreatetruecolor(450, 30);
			$bgc = imagecolorallocate($im, 255, 255, 255);
			$tc  = imagecolorallocate($im, 0, 0, 0);
	
			imagefilledrectangle($im, 0, 0, 450, 30, $bgc);
	
			/* Output an error message */
			imagestring($im, 1, 5, 5, 'Error loading ' . $pv_Template, $tc);
			imagepng($im);
			imagedestroy($im);
		
		} else {
	
			$fv_ImageHeight = imagesy($fv_TemplateImage);
			$fv_ImageWidth = imagesx($fv_TemplateImage);
			
		//	Create the new image and set up transparancy
			$fv_colorizedImage = imagecreatetruecolor ($fv_ImageWidth, $fv_ImageHeight);
			$fv_TransparantIndex = imagecolorallocate ($fv_colorizedImage, $pv_Red, $pv_Green, $pv_Blue);
			imagecolortransparent($fv_colorizedImage, $fv_TransparantIndex);   
		   
		//	Copy the template, preserving transparancy   
			imagecopymerge ($fv_colorizedImage, $fv_TemplateImage, 0, 0, 0, 0, $fv_ImageWidth, $fv_ImageHeight, 100);
		   
		//	Colorize the image
			imagefilter($fv_colorizedImage, IMG_FILTER_COLORIZE,  $pv_Red, $pv_Green, $pv_Blue);
		
		//	Output the image
			header("Content-type: $gv_MIME");
			$last_modified_time = filemtime('colorize.php'); 
			header("Last-Modified: ".gmdate("D, d M Y H:i:s", $last_modified_time)." GMT");
			header('Expires: ' . gmdate('D, d M Y H:i:s', time()+24*60*60) . ' GMT');

	switch ($gv_MIME) {
		case 'image/png':
			imagepng($fv_colorizedImage);
		break;
		
		case 'image/jpeg':
			imagejpeg($fv_colorizedImage);
		break;
		
		case 'image/gif':
			imagegif($fv_colorizedImage);
		break;
	}


			
		//	Clean up
			imagedestroy($fv_colorizedImage);
			imagedestroy($fv_TemplateImage);
	}
}

function m_Hex2int($pv_HexadecimalColor) {
/*	Parameter:	$pv_HexadecimalColor (string)	6-digit hexadecimal color
	Returns:		array			3 elements 'red', 'green', & 'blue' = integer color values, 0 - 255
 				Converts a 6 digit hexadecimal number into an array of
				3 integer values ('red'  => red value, 'green'  => green, 'blue'  => blue)
*/
	return array(
		'red' => hexdec(substr($pv_HexadecimalColor, 0, 2)), // 1st pair of digits
		'green' => hexdec(substr($pv_HexadecimalColor, 2, 2)), // 2nd pair
		'blue' => hexdec(substr($pv_HexadecimalColor, 4, 2))  // 3rd pair
	);
}
function m_ValidHexColor($pv_HexadecimalColor = 'cccccc', $pv_DefaultColor = 'cccccc') {

/*	Parameters:	$input (string)		6-digit hexadecimal string to be validated
				$pv_DefaultColor (string)	default color to be returned if $input isn't valid
	Returns:		string				a valid 6-digit hexadecimal color
 				Returns $input if it is a valid hexadecimal color, 
				otherwise returns $default (which defaults to grey).
				A valid Hexadecimal color is exactly 6 characters long
				and composed of numerals or letters from a to f.
 */ 
 
 $pv_HexadecimalColor = trim($pv_HexadecimalColor, '#');
 
 
//    return (eregi('^[0-9a-f]{6}$', $pv_HexadecimalColor)) ? $pv_HexadecimalColor : $pv_DefaultColor;
//		eregi deprecated ~sigh~
    return (preg_match('/^[0-9a-f]{6}$/i', $pv_HexadecimalColor)) ? $pv_HexadecimalColor : $pv_DefaultColor;
}

function m_SafeString($pv_UserInput) {
    return (isset($pv_UserInput))?stripslashes($pv_UserInput):'template.png';
}

?>