<?php
/* -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+----- 

	/system/core/25_sessions.inc
	
		Callback functions for session_set_save_handler() allowing Molly to store 
		sessions and associated information in a database table.  Among other 
		things this allows us to easily determine who's logged in, and to share 
		sessions across multiple web servers.
	
   -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+----- */
   
/*

The SQL:

	CREATE TABLE IF NOT EXISTS `molly_sessions` (
	  `sesskey` varchar(32) NOT NULL default '',
	  `expiry` int(10) unsigned NOT NULL default '0',
	  `value` text NOT NULL,
	  `loginname` varchar(255) NOT NULL default '',
	  `user_id` mediumint(9) default '0',
	  PRIMARY KEY  (`sesskey`)
	) ENGINE=MyISAM DEFAULT CHARSET=latin1;

*/


$GLOBALS['gv_SessionLifetime'] = ini_get("session.gc_maxlifetime");

function m_SessionQuery($pv_Query){
	/* Configured in molly.ini - multiple web servers can share a single session by connecting to the same database server */

	$fv_SessionDBHost  = $GLOBALS['gv_SiteGlobals']['Sessions']['SessionHost'];
	$fv_SessionDBName  = $GLOBALS['gv_SiteGlobals']['Sessions']['SessionDatabase'];
	$fv_SessionDBUserName  = $GLOBALS['gv_SiteGlobals']['Sessions']['SessionUser'];
	$fv_SessionDBPassword  =  $GLOBALS['gv_SiteGlobals']['Sessions']['SessionPassword'];
	$fv_SessionTable = $GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable'];
	
	return m_DoQuery($pv_Query, $fv_SessionDBHost, $fv_SessionDBUserName, $fv_SessionDBPassword, $fv_SessionDBName);
}



function m_session_open($pv_SavePath, $pv_SessionName) {
	global $gv_SessionLifetime;

	 $fv_Query = "DELETE FROM {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} WHERE expiry < " . time() .";";
	 $fo_QueryResult = m_SessionQuery($fv_Query);
	return TRUE;
} 

/* This function will be registered to close session handling.	Normally,
   this means finishing all writes and closing the session store on the
   filesystem.	This function does nothing since we close the database
   connection after every query.
*/
									 
function m_session_close() {
	 return TRUE;
}



/* This function will retrieve session variables for a particular session key.	Each
   time a session is read from the database, this function "freshens" the session
   so that it will not be removed by garbage collection.  PHP handles de-serializing
   the data for you.
*/
function m_session_read($pv_Key) {
	 global $gv_SessionLifetime;

	 $fv_Query = "SELECT value FROM {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} WHERE sesskey = '$pv_Key' AND expiry > " . time().";";
	 $fo_QueryResult = m_SessionQuery($fv_Query);

	 if ($fo_QueryResult != false) {
		  $pv_SessionData = $fo_QueryResult->GetRows();
		  if(count($pv_SessionData) == 0) {
			return '';
		  }
		  $fv_SessionExpiration = time() + $gv_SessionLifetime;
		  $fv_Query = "UPDATE {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} SET expiry = $fv_SessionExpiration WHERE sesskey = '$pv_Key';";
		  $fo_QueryResult = m_SessionQuery($fv_Query);
		  return $pv_SessionData[0]['value'];
	 } else {
		 return "";
	 }
}



/* This function stores the session data for a particular key.	If the key
   already exists in the table, it updates the key's value and expiry time.
   PHP serializes the session data and sends it in as a string.
*/
function m_session_write($pv_Key, $pv_SessionData) {
	global $gv_SessionLifetime;
	
	$fv_SessionExpiration = time() + $gv_SessionLifetime;
	$pv_SessionData = addslashes($pv_SessionData);
	if(isset($_SESSION['s_LoginName'])) {
		$fv_loginName=$_SESSION['s_LoginName'];
	} else {
		$fv_loginName = '';
	}
	
	if (isset($_SESSION['s_UserID'])) {
		$fv_UserID=$_SESSION['s_UserID'];
	} else {
		$fv_UserID='NULL';
	}
	
	$fv_Query = "INSERT INTO {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} VALUES ('$pv_Key', $fv_SessionExpiration, '$pv_SessionData', '$fv_loginName', $fv_UserID);";
	$fo_QueryResult = m_SessionQuery($fv_Query);

	if (!$fo_QueryResult) {
		$fv_Query = "UPDATE {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} SET expiry = $fv_SessionExpiration, value = '$pv_SessionData',  loginname='$fv_loginName', user_id=$fv_UserID WHERE sesskey = '$pv_Key' AND expiry > " . time().";";
		$fo_QueryResult = m_SessionQuery($fv_Query);
	}
	return ($fo_QueryResult != false);
}



/* Destroys a session by deleting its record from the table */
function m_session_destroy($pv_Key) {

	 $fv_Query = "DELETE FROM {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} WHERE sesskey = '$pv_Key';";
	 $fo_QueryResult = m_SessionQuery($fv_Query);
	 return $fo_QueryResult;
}



/* This is the function which collects garbage.	 The probability that PHP
   will run this function is set in the php.ini directive
   "session.gc_probability"
*/
function m_session_gc($maxlifetime) {

	 $fv_Query = "DELETE FROM {$GLOBALS['gv_SiteGlobals']['Sessions']['SessionTable']} WHERE expiry < " . time() .";";
	 $fo_QueryResult = m_SessionQuery($fv_Query);
	 
	 return true;
}

/*
 This registers the above functions as the handlers for the various operations
   on session data.
*/
session_set_save_handler
	 (
		  "m_session_open",
		  "m_session_close",
		  "m_session_read",
		  "m_session_write",
		  "m_session_destroy",
		  "m_session_gc"
	 );
session_name($GLOBALS['gv_SiteGlobals']['Sessions']['SessionName']);
session_start();   
?>