<?php

// Query to increase size of password field:
// ALTER TABLE `molly_login` CHANGE `password` `password` VARCHAR( 512 ) CHARACTER SET latin1 COLLATE latin1_bin NOT NULL DEFAULT ''

	m_AutoInclude(MOLLY_SYSTEM_DIRECTORY . 'core/security/','inc');

	if (!isset($_SESSION['tokens']['LoggedIn'])){
		$_SESSION['tokens']['LoggedIn'] = 0;
		$_SESSION['tokens']['LastName'] = '';
		$_SESSION['tokens']['FirstName'] = '';
		$_SESSION['tokens']['Salutation'] = '';
		$_SESSION['tokens']['UserID'] = '';
		$_SESSION['tokens']['LoginName'] = '';
	}

	function m_clean_params_for_query($pv_QueryComponent) {
		global $gv_SiteGlobals, $ADODB_FETCH_MODE;

		// Connect to ADOdb
		$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
		$pv_HostName=$gv_SiteGlobals['Database']['DBHostName'];
		$pv_User=$gv_SiteGlobals['Database']['DBUserName'];
		$pv_Password=$gv_SiteGlobals['Database']['DBPassword'];
		$pv_Database=$gv_SiteGlobals['Database']['DatabaseName'];
		$pv_DatabaseApp=$gv_SiteGlobals['Database']['DatabaseApp'];
		$fo_DataBaseConnection = ADONewConnection($pv_DatabaseApp);
		$fo_DataBaseConnection->debug = $gv_SiteGlobals['Database']['debug'];

		// Throw away anything after first semi-colon or comment
		//$fv_StringPieces = explode(';', $pv_QueryComponent);
		//$fv_StringPieces = explode('--', $fv_StringPieces[0]);
		$fv_StringPieces = explode('--', $pv_QueryComponent);
		$fv_BaseString = $fv_StringPieces[0];

		// Escape any remaining problematic characters
//		$fv_CleanString = str_replace(array('\\', "\0", "\n", "\r", "'", '"', "\x1a", '</script>', '<script>', '<'), array('\\\\', '\\0', '\\n', '\\r', "\\'", '\\"', '\\Z', '','',''), $fv_BaseString);
		$fv_CleanString = str_replace($gv_SiteGlobals['Security']['QueryStringAttacks'],$gv_SiteGlobals['Security']['QueryStringReplacements'], $fv_BaseString);

		// Run through ADOdb's cleaner
		$fv_CleanString = substr($fo_DataBaseConnection->qStr($fv_CleanString), 1, -1);

		return $fv_CleanString;
	}

	function m_clean_array($pv_input) {
		// cleans input, recursing through arrays
		if(is_array($pv_input)) {
			foreach($pv_input as $fv_key=>$fv_val) {
				$pv_input[$fv_key] = m_clean_array($fv_val);
			}
			return $pv_input;
		} else {
			return m_clean_params_for_query($pv_input);
		}
	}

	function m_HashPassword($pv_password, $pv_algorithm = null, $pv_salt = null, $pv_iterations = null) {
		// Returns hashed password or FALSE if failure (such as an invalid algorithm)
		// This SHOULD NOT be used with an == for comparing to a hash in the database; use the function designed for that

		global $gv_SiteGlobals;

		$fv_algo = strtolower(isset($pv_algorithm) ? $pv_algorithm : $gv_SiteGlobals['Login']['PasswordEncryption']);

		if($fv_algo == 'none') {
			return $pv_password;
		} else if($fv_algo == 'auto') {
			return password_hash($pv_password, PASSWORD_DEFAULT);
		} else if($fv_algo == 'bcrypt' || $fv_algo == 'blowfish') {
			return password_hash($pv_password, PASSWORD_BCRYPT);
		} else if($fv_algo == 'argon2i' && defined('PASSWORD_ARGON2I')) {
			// Argon2i was introduced in PHP 7.2, this (^) does a safety check before we try to use it
			return password_hash($pv_password, PASSWORD_DEFAULT);
		} else if(in_array($fv_algo, hash_algos())) {
			return $fv_algo . ':' . hash($fv_algo, $pv_password);
		} else if(strpos($fv_algo, 'hmac_') === 0 ? in_array(substr($fv_algo, 5), (function_exists('hash_hmac_algos') ? hash_hmac_algos() : hash_algos())) : FALSE) {
			$fv_salt = bin2hex(isset($pv_salt) ? $pv_salt : random_bytes(32));
			return $fv_algo . ':' . $fv_salt . ':' . hash_hmac(substr($fv_algo, 5), $pv_password, $fv_salt);
		/*} else if(strpos($pv_algorithm, 'pbkdf2_') === 0 ? in_array(substr($pv_algorithm, 7), (function_exists('hash_hmac_algos') ? hash_hmac_algos() : hash_algos())) : FALSE) {
			$fv_salt = (isset($pv_salt) ? $pv_salt : random_bytes(64));
			$fv_iterations = (isset($pv_iterations) ? $pv_iterations : 40000); // 40,000 was arbitrarily decided as a safe number based on recommendation. NIST recommends 10,000 at minimum
			return hash_pbkdf2(substr($pv_algorithm, 7), $pv_password, $fv_salt, $fv_iterations);
		*/
		} else if($fv_algo == 'pbkdf2') {
			return m_PBKDF2($pv_password);
		}

		return FALSE;
	}

	function m_CheckHashDefault($pv_password, $pv_hashed, $pv_algorithm) {
		// Returns boolean value of whether the password matched the hashed password

		global $gv_SiteGlobals;

		$fv_algo = strtolower(isset($pv_algorithm) ? $pv_algorithm : $gv_SiteGlobals['Login']['PasswordEncryption']);

		if($fv_algo == 'none') {
			return hash_equals($pv_password, $pv_hashed); // hash_equals is just a timing-attack-safe == for strings
		} else if($fv_algo == 'auto') {
			if (password_verify($pv_password, $pv_hashed)) {
				if (password_needs_rehash($pv_hashed, PASSWORD_DEFAULT)) {
					$newHash = password_hash($pv_password, PASSWORD_DEFAULT);
					m_DoQuery("UPDATE `$fv_Table` SET `$fv_PasswordField` = '$fv_hashed' WHERE `$fv_NameField`='$pv_LoginName'");
				}
				return TRUE;
			}
		} else if($fv_algo == 'bcrypt' || $fv_algo == 'blowfish') {
			$fv_hash_info = password_get_info($pv_hashed);
			return $fv_hash_info['algo'] == PASSWORD_BCRYPT && password_verify($pv_password, $pv_hashed);
		} else if($fv_algo == 'argon2i' && defined('PASSWORD_ARGON2I')) {
			// Argon2i was introduced in PHP 7.2, this (^) does a safety check before we try to use it
			$fv_hash_info = password_get_info($pv_hashed);
			return $fv_hash_info['algo'] == PASSWORD_ARGON2I && password_verify($pv_password, $pv_hashed);
		} else if(in_array($fv_algo, hash_algos())) {
			$fv_info = explode(':', $pv_hashed);
			return hash_equals(hash($fv_algo, $pv_password), $fv_info[1]);
		} else if(strpos($fv_algo, 'hmac_') === 0 ? in_array(substr($fv_algo, 5), (function_exists('hash_hmac_algos') ? hash_hmac_algos() : hash_algos())) : FALSE) {
			$fv_parts = explode(':', $pv_hashed);
			return hash_equals(hash_hmac(substr($fv_parts[0], 5), $pv_password, $fv_parts[1]), $fv_parts[2]);
		} else if(/* strpos($pv_algorithm, 'pbkdf2_') === 0 || */ $fv_algo == 'pbkdf2') {
			return m_PBKDF2_Verify($pv_password, $pv_hashed);
		}

		return FALSE;
	}

	function m_CheckHashBrute($pv_password, $pv_hashed) {
		// Returns boolean value of whether the password matched the hashed password
		// Tries all hashing methods it can detect

		global $gv_SiteGlobals;

		if((strpos($pv_password, ':') === FALSE && strpos($pv_password, '$') === FALSE && strlen($pv_password) > 0) ? hash_equals($pv_password, $pv_hashed) : FALSE) return TRUE; // hash_equals is just a timing-attack-safe == for strings
		if(password_verify($pv_password, $pv_hashed)) return TRUE; // will check both bcrypt/blowfish & argon2i (if PHP >= 7.2)

		if(strpos($pv_hashed, ':') !== FALSE) {
			$fv_parts = explode(':', $pv_hashed);
			if(count($fv_parts) == 2 && in_array($fv_parts[0], hash_algos())) {
				if(hash_equals(hash($fv_parts[0], $pv_password), $fv_parts[1])) return TRUE;
			} else if(count($fv_parts) == 3 && strpos($fv_parts[0], 'hmac_') === 0 ? in_array(substr($fv_parts[0], 5), (function_exists('hash_hmac_algos') ? hash_hmac_algos() : hash_algos())) : FALSE) {
				if(hash_equals(hash_hmac(substr($fv_parts[0], 5), $pv_password, $fv_parts[1]), $fv_parts[2])) return TRUE;
			} else if(count($fv_parts) == 5) {
				if(m_PBKDF2_Verify($pv_password, $pv_hashed)) return TRUE;
			}
		}

		// ELSE - CHECK Drupal & WordPress
		// https://cgit.drupalcode.org/drupal/tree/core/lib/Drupal/Core/Password/PhpassHashedPassword.php#n223
		// https://build.trac.wordpress.org/browser/branches/4.9/wp-includes/class-phpass.php#L121

		/*$fv_pass = $pv_password;
		// Drupal prepends $S$, $H$, & $P$ with a U (e.g. U$S$) if the password should get an extra md5 before normal hashing
		if($pv_hashed[0] == 'U') {
			$fv_pass = hash('md5', $fv_pass);
			$pv_hashed = substr($pv_hashed, 1);
		}
		*/

		// Drupal SHA512 - $S$

		// $P$ (WordPress, Drupal), $H$ (phpBB, WordPress, Drupal)

		return FALSE;
	}

	function m_ValidateAgainstDB($pv_LoginName, $pv_Password) {
		global $gv_SiteGlobals;

		// return user_id for successful login, FALSE for failed login.

		foreach ($gv_SiteGlobals['Login']['LoginDBHost'] as $fv_Index => $fv_Host) {
			$fv_DBHost = $gv_SiteGlobals['Login']['LoginDBHost'][$fv_Index];
			$fv_Database = $gv_SiteGlobals['Login']['LoginDatabase'][$fv_Index];
			$fv_UserName = $gv_SiteGlobals['Login']['LoginDBUserName'][$fv_Index];
			$fv_Password = $gv_SiteGlobals['Login']['LoginDBPassword'][$fv_Index];
			$fv_Table = $gv_SiteGlobals['Login']['LoginTable'][$fv_Index];
			$fv_NameField = $gv_SiteGlobals['Login']['LoginNameField'][$fv_Index];
			$fv_IDField = $gv_SiteGlobals['Login']['LoginIDField'][$fv_Index];
			$fv_PasswordField = $gv_SiteGlobals['Login']['LoginPasswordField'][$fv_Index];
			$fv_PasswordEncryption = (isset($gv_SiteGlobals['Login']['LoginPasswordEncryption'][$fv_Index]) ? $gv_SiteGlobals['Login']['LoginPasswordEncryption'][$fv_Index] : $gv_SiteGlobals['Login']['PasswordEncryption']);

			$fv_Query = "SELECT * FROM `$fv_Table` WHERE `$fv_NameField`='$pv_LoginName'";
			$fv_Results = m_FetchDBHash($fv_Query, $fv_DBHost, $fv_UserName, $fv_Password, $fv_Database);
			if (count($fv_Results) == 1) {
				$fv_validLogin = FALSE;
				if(m_CheckHashDefault($pv_Password, $fv_Results[0][$fv_PasswordField], $fv_PasswordEncryption)) {
					$fv_validLogin = TRUE;
				} else {
					$fv_brute = FALSE;
					$fv_rehash = FALSE;
					if(isset($gv_SiteGlobals['Login']['LoginPasswordEncryption'][$fv_Index]) ? $gv_SiteGlobals['Login']['LoginPasswordEncryption'][$fv_Index] == 'integrate' : FALSE) {
						$fv_brute = TRUE;
					} else if(isset($gv_SiteGlobals['Login']['LoginOldPasswordHandling'][$fv_Index]) ? $gv_SiteGlobals['Login']['LoginOldPasswordHandling'][$fv_Index] == 'rehash' : FALSE) {
						$fv_brute = TRUE;
						$fv_rehash = TRUE;
					}

					if($fv_brute ? m_CheckHashBrute($pv_Password, $fv_Results[0][$fv_PasswordField]) : FALSE) {
						if($fv_rehash) {
							// UPDATE (REHASH)
							$fv_hashed = m_HashPassword($pv_Password, $fv_PasswordEncryption);
							m_DoQuery("UPDATE `$fv_Table` SET `$fv_PasswordField` = '$fv_hashed' WHERE `$fv_NameField`='$pv_LoginName'");
						}

						$fv_validLogin = TRUE;
					}
				}

				if($fv_validLogin === TRUE) {
					$_SESSION['s_LoginName'] = $pv_LoginName;
					$fv_UserID =  $fv_Results[0]["$fv_IDField"];
					$fv_Query = "update `$fv_Table` set login_time=now() where user_id=$fv_UserID;"; //Timestamp when user logged in
					$fv_Results = m_DoQuery($fv_Query);
					return $fv_UserID;
				}
			}
		}
		return FALSE;
	}

	function m_ValidateAgainstLDAP($pv_LoginName, $pv_Password) {
		global $gv_SiteGlobals;

		// return user_id for successful login, FALSE for failed login.
		foreach ($gv_SiteGlobals['ldap']['ServerAddress'] as $fv_Index => $fv_Host) {
			$fv_LDAPConnection = @ldap_connect($fv_Host, $gv_SiteGlobals['ldap']['Port'][$fv_Index]);
			if($fv_LDAPConnection) { //connected
				ldap_set_option($fv_LDAPConnection, LDAP_OPT_PROTOCOL_VERSION, $gv_SiteGlobals['ldap']['LdapVersion'][$fv_Index]);
				if (ldap_bind($fv_LDAPConnection, "uid=$pv_LoginName,{$gv_SiteGlobals['ldap']['BaseDN'][$fv_Index]}", $pv_Password) ) {
					// OK, they're authenticated with LDAP, now validate with Molly
					foreach ($gv_SiteGlobals['Login']['LoginDBHost'] as $fv_Index => $fv_Host) {
						$fv_DBHost = $gv_SiteGlobals['Login']['LoginDBHost'][$fv_Index];
						$fv_Database = $gv_SiteGlobals['Login']['LoginDatabase'][$fv_Index];
						$fv_UserName = $gv_SiteGlobals['Login']['LoginDBUserName'][$fv_Index];
						$fv_Password = $gv_SiteGlobals['Login']['LoginDBPassword'][$fv_Index];
						$fv_Table = $gv_SiteGlobals['Login']['LoginTable'][$fv_Index];
						$fv_NameField = $gv_SiteGlobals['Login']['LoginNameField'][$fv_Index];
						$fv_IDField = $gv_SiteGlobals['Login']['LoginIDField'][$fv_Index];
						//$fv_PasswordField = $gv_SiteGlobals['Login']['LoginPasswordField'][$fv_Index];

						$fv_Query = "SELECT $fv_IDField FROM `$fv_Table` WHERE $fv_NameField='$pv_LoginName'";
						$fv_Results = m_FetchDBHash($fv_Query, $fv_DBHost, $fv_UserName, $fv_Password, $fv_Database);
						if ($fv_Results) {
							$_SESSION['s_LoginName'] = $pv_LoginName;
							$fv_UserID =  $fv_Results[0]["$fv_IDField"];
							return $fv_UserID;
						} else {
							// need to create a record for this newly ldap-validated user
							$fv_Query ="INSERT INTO `$fv_Database`.`$fv_Table` (`$fv_IDField`, `loginname`, `password`) VALUES (NULL, '$pv_LoginName', 'LDAP');";
							$fv_Results = m_DoQuery($fv_Query);
							$fv_Query = "SELECT $fv_IDField FROM `$fv_Table` WHERE $fv_NameField='$pv_LoginName'";
							$fv_Results = m_FetchDBHash($fv_Query, $fv_DBHost, $fv_UserName, $fv_Password, $fv_Database);
							if ($fv_Results) {
								$_SESSION['s_LoginName'] = $pv_LoginName;
								$fv_UserID =  $fv_Results[0]["$fv_IDField"];
								$fv_filter="uid=$pv_LoginName";
								$fv_TheseValues = array("sn", "givenName", "ritEduMiddleInitial", "title", "mail", "ritEduAccountType");
								$fv_SearchResults=ldap_search($fv_LDAPConnection, $gv_SiteGlobals['ldap']['BaseDN'][$fv_Index], $fv_filter, $fv_TheseValues);
								$fv_Info = ldap_get_entries($fv_LDAPConnection, $fv_SearchResults);

								$fv_FirstName = $fv_Info[0]['givenname'][0];
								$fv_LastName = $fv_Info[0]['sn'][0];
								$fv_MiddleInitial = $fv_Info[0]['ritedumiddleinitial'][0];
								//$fv_Classification = $fv_Info[0]['title'][0];
								$fv_EMail = $fv_Info[0]['mail'][0];

								$fv_Query = "INSERT INTO `molly_username` (`user_id`, `lastname`, `firstname`, `middlename`, `salutation`, `suffix`, `email`, `preferredlanguage`, `login_time`) VALUES ($fv_UserID, '$fv_LastName', '$fv_FirstName', '$fv_MiddleInitial', NULL, NULL, '$fv_EMail', 'en','')";
								$fv_Results = m_DoQuery($fv_Query);

								return $fv_UserID;
							}
						}
					}
				} else {
					$LDAP_OPT_DIAGNOSTIC_MESSAGE=0x0032;
					    if (ldap_get_option($fv_LDAPConnection, $LDAP_OPT_DIAGNOSTIC_MESSAGE, $extended_error)) {

    				m_debug("bad user/pass combo<br />Error Binding to LDAP: $extended_error<br />$fv_LDAPConnection<!--<p>Login may be broken due to ITS maintenance.<br />We are working to resolve the issue as soon as possible.  Thank you for your patience!</p>-->");
    				} else {
    				m_debug("bad user/pass combo");
    				}
    			}
			} else {
				m_debug('no connection');
			}
		}
		return FALSE; // not found in LDAP
	}

	function m_CheckLogin($pv_LoginName, $pv_Password) {
		global $gv_SiteGlobals;

		if ($pv_Password != "") {
			switch ($gv_SiteGlobals['Login']['Method']) {
				case 'ldap':
					return m_ValidateAgainstLDAP($pv_LoginName, $pv_Password);
					break;
				default:
				case 'db':
					return m_ValidateAgainstDB($pv_LoginName, $pv_Password);
					break;
			}
		} else {
			return FALSE;
		}
	}

	function m_DoLogin($pv_UserID) {
		global $gv_SiteGlobals;

		$fv_Query = "SELECT * FROM `molly_username` WHERE user_id='$pv_UserID'";
		$fv_Results = m_FetchDBHash($fv_Query);

		$_SESSION['s_LastName'] 	= $fv_Results[0]['lastname'];
		$_SESSION['s_FirstName'] 	= $fv_Results[0]['firstname'];
		$_SESSION['s_Salutation'] 	= $fv_Results[0]['salutation'];
		$_SESSION['s_UserID'] 		= $fv_Results[0]['user_id'];

		$_SESSION['tokens']['LoggedIn'] = 1;
		$_SESSION['tokens']['LastName'] = $_SESSION['s_LastName'];
		$_SESSION['tokens']['FirstName'] = $_SESSION['s_FirstName'];
		$_SESSION['tokens']['Salutation'] = $_SESSION['s_Salutation'];
		$_SESSION['tokens']['UserID'] = $_SESSION['s_UserID'];
		$_SESSION['tokens']['LoginName'] = $_SESSION['s_LoginName'];

		$fv_Query = "update `molly_username` set login_time=now() where user_id=$pv_UserID;"; //Timestamp when user logged in
		$fv_Results = m_DoQuery($fv_Query);

		if ($gv_SiteGlobals['Login']['TrackLogins'] == 'on') {
			if ($pv_UserID > $gv_SiteGlobals['Login']['TrackUserIDsAbove']) {
					// insert a record into login tracking table
					$fv_Query = "INSERT INTO `molly_login_history` (`user_id`, `login_time`) VALUES ('$pv_UserID', CURRENT_TIMESTAMP)";
					$fv_Results = m_DoQuery($fv_Query);
				}
		}
	}

	function m_permitted($pv_GroupList) {
		$pv_GroupList = str_replace(', ', ',', $pv_GroupList);
		$pv_GroupList = str_replace(' ', ',', $pv_GroupList);
		$fv_boom = explode(',', $pv_GroupList);
		foreach($fv_boom as $pv_Group) {
			if (isset($_SESSION['s_UserID'])) {
				if (strtolower($pv_Group)=='all') {
					return TRUE;
				} else {
					$fv_UserID = $_SESSION['s_UserID'];
					$fv_Query = "SELECT * FROM `molly_permissions` WHERE user_id='$fv_UserID' AND group_name='$pv_Group'";
					$fv_Results = m_FetchDBHash($fv_Query);
					//return count($fv_Results);
					if(count($fv_Results) > 0) {
						if($fv_Results[0]['permitted'] == true) {
							return TRUE;
						}
					}
				}
			} else {
				if($pv_Group=="guest") {
					return TRUE;
				}
			}
		}
		return false;
	}

	/* Legacy Code from early 2000s Department Web Site Disabled Due To Uncertainty Over Security
	function m_Jasmine($p_StringToEncrypt) {
                settype($v_EncryptedString,"string");
                $n = strlen($p_StringToEncrypt);

                $v_working = array();

                // first convert to numbers
                for ($i=0;$i<$n;$i++) {
                                $v_working[$i] = Ord($p_StringToEncrypt[$i]);
                }

                // now hash
                for ($i=0;$i<$n-1;$i++) {
                                if ( $i % 2 != 0) {
                                   $v_working[$i] = ($v_working[$i] * $v_working[$i+1]) % 255;
                                } else {
                                                $v_working[$i] = ($v_working[$i] + $v_working[$i+1]) % 255;
                                }
                }

                $v_working[$n-1] = ( $v_working[$n-1] * $v_working[$n-1] /($v_working[0] + 1) ) % 255;

                $v_EncryptedString = "";
                for ($i=0;$i<$n;$i++) {
                                $v_EncryptedString = $v_EncryptedString . sprintf("%03s", $v_working[$i]);
                }
                return $v_EncryptedString;
	}*/

// Clean $_THING variables; aka, don't trust user input
$_GET = m_clean_array($_GET);
$_POST = m_clean_array($_POST);
$_REQUEST = m_clean_array($_REQUEST);

?>
