<?php

function m_PullParse($pv_XMLString) {
	// These are used for indicating if something should not be rendered
	// Using a global stack to prevent issues with multiple calls to m_PullParse (<maml:include> does this)
	// It shouldn't be needed, but it's a safety measure
	// To stop pass the element class name to $gv_stopParse and the parser will pick it up
	// If there are items in the $gv_stopParseStack stack, parsing is not active
	//     Aka, count($gv_stopParseStack) == 0 indicates parsing is active
	// $gv_stopParseOffStackStack is used for the special <maml:else> case to hold the existing stack
	// Yes, $gv_stopParseOffStackStack is an ugly name
	// It's supposed to mean the "off-stack" ("off" as in alternate) stack for when not on $gv_stopParseStack
	global $gv_stopParse, $gv_stopParseStack, $gv_stopParseOffStackStack;
	
	// Initialize globals if do not exist
	if(!isset($gv_stopParse)) {
		$gv_stopParse = "";
	}
	if(!isset($gv_stopParseStack)) {
		$gv_stopParseStack = array();
	}
	if(!isset($gv_stopParseOffStackStack)) {
		$gv_stopParseOffStackStack = array();
	}
	
	// Stack for tracking Molly tags so can call end() when they finish
	$fv_TagObjectArray = array();
	$fv_CurrentTagIndex = -1;
	
	// Create XMLReader
	$fo_CurrentNode = new XMLReader();
	
	// Load XML passed to function
	$fv_PageText = str_replace ('&','&amp;' ,$pv_XMLString); //hack entities: turn every & into an &amp;  (Heather made me do it ;o)
	$fo_CurrentNode->XML($fv_PageText); //Load it into the reader object
	
	// XMLReader Settings
	$fo_CurrentNode->setParserProperty(XMLReader::SUBST_ENTITIES, FALSE);
	$fo_CurrentNode->setParserProperty(XMLReader::LOADDTD,  FALSE); // Don't check against DTD - too slow & a security issue (see: https://www.owasp.org/index.php/XML_External_Entity_(XXE)_Processing )
	$fo_CurrentNode->setParserProperty(XMLReader::DEFAULTATTRS,  FALSE); //Don't Load DTD and default attributes but do not validate
	$fo_CurrentNode->setParserProperty(XMLReader::VALIDATE, FALSE); // Don't validate - too slow
	
	// Loop through all XML elements & process by type
	while ($fo_CurrentNode->read()) {
		switch ($fo_CurrentNode->nodeType) {
			
			case XMLReader::ELEMENT:
				// XML start tags
				
				// Setup namespace (defaulting to HTML if none)
				if (!$fo_CurrentNode->prefix) {
					$fv_NameSpace = "html";
				} else {
					$fv_NameSpace = $fo_CurrentNode->prefix;
				}
				
				// Generate class name
				$fv_TagName = $fv_NameSpace . '_' . $fo_CurrentNode->localName;
				
				// Check if has attributes
				if ($fo_CurrentNode->isEmptyElement) { // have to check this *before* moving to attributes
					$fv_TagIsEmpty = TRUE;
				} else {
					$fv_TagIsEmpty = FALSE;
				}
				
				// Process Molly tags
				if (m_IsMollyTag($fv_TagName)) {
					$fv_TagMolly = new $fv_TagName;
					
					// Load attributes to precheck can check permitted
					if ($fo_CurrentNode->hasAttributes) { // load the object with it's attributes
						while ($fo_CurrentNode->moveToNextAttribute()) {
							$fv_AttributeName = $fo_CurrentNode->name;
							$fv_AttributeValue =  $fo_CurrentNode->value;
							$fv_TagMolly->$fv_AttributeName = $fv_AttributeValue;
						}
					}
					
					// Ask tag if should stop parsing
					if(count($gv_stopParseStack) == 0) {
						$fv_TagMolly->precheck();
					} else if(count($gv_stopParseStack) == 1 && $fv_TagName == "maml_else") {
						// Special edge case for <maml:else>
						$fv_TagMolly->precheck();
					}
						
					// Check parsing status
					if($gv_stopParse != "") {
						// Stop parse requested by precheck
						if(!$fv_TagIsEmpty) {
							array_push($gv_stopParseStack, $gv_stopParse);
							$gv_stopParse = "";
						}
					} else if(count($gv_stopParseStack) > 0) {
						// Parsing is already stopped
						if(!$fv_TagIsEmpty) {
							array_push($gv_stopParseStack, $fv_TagName);
						}
					} else {
						// Parsing is active & stopping has not been requested
						// (So parse the tags)
						$fv_TagObjectArray[] = $fv_TagMolly; // create a new tag object on the stack - destroy on end tag
						$fv_CurrentTagIndex = $fv_CurrentTagIndex + 1;
						
						$fv_TagObjectArray[$fv_CurrentTagIndex]->begin();
						
						if($fv_TagIsEmpty) {
							$fv_TagObjectArray[$fv_CurrentTagIndex]->end();		
							array_pop($fv_TagObjectArray); //delete this tag's object from the stack
							$fv_CurrentTagIndex = $fv_CurrentTagIndex - 1;
						}
					}
				} else {
					// Check if parsing
					if(count($gv_stopParseStack) == 0) {
						// Parsing is active, so parse
						echo "<" . $fo_CurrentNode->localName;
						if ($fo_CurrentNode->hasAttributes) { // load the object with it's attributes
							while ($fo_CurrentNode->moveToNextAttribute()) {
								$fv_AttributeName = $fo_CurrentNode->name;
								$fv_AttributeValue =  $fo_CurrentNode->value;
								echo " $fv_AttributeName = \"$fv_AttributeValue\"";
							}
						}
						
						if ($fv_TagIsEmpty) {
							echo " /";
						}
						echo ">";
					} else {
						if(!$fv_TagIsEmpty) {
							// Add to stack if not parsing
							array_push($gv_stopParseStack, $fv_TagName);
						}
					}
				}
			break;
			
			case XMLReader::END_ELEMENT:
				// XML end tags
				
				// Check if parsing
				if(count($gv_stopParseStack) > 0) {
					// If parsing stopped, remove from stack
					array_pop($gv_stopParseStack);
				} else {
					// Parsing active, so parse
					
					// Generate namespace (default html) & class name
					if (!$fo_CurrentNode->prefix) {
						$fv_NameSpace = "html";
					} else {
						$fv_NameSpace = $fo_CurrentNode->prefix;
					}
					$fv_TagName = $fv_NameSpace . '_' . $fo_CurrentNode->localName;
		
					if (m_IsMollyTag($fv_TagName)) {
						// Run the Molly tag's end() & remove from Molly tags stack
						$fv_TagObjectArray[$fv_CurrentTagIndex]->end();		
						array_pop($fv_TagObjectArray);
						$fv_CurrentTagIndex = $fv_CurrentTagIndex - 1;
					} else {
						// Echo closing tag if HTML
						echo "</" . $fo_CurrentNode->localName . ">";
					}
				}
			break;
			
			case XMLReader::TEXT:
			case XMLReader::CDATA:
			case XMLReader::WHITESPACE:
			case XMLReader::SIGNIFICANT_WHITESPACE:
				if(count($gv_stopParseStack) == 0) {
					// Print if parsing
					print $fo_CurrentNode->value;
				}
			break;
			
			case XMLReader::PI: // Processing Instruction (PHP)
				if(count($gv_stopParseStack) == 0) {
					// Run PHP if parsing
					global $gv_InRow;
					if ($gv_InRow) {
						echo $fo_CurrentNode->value;
					} else {
						eval($fo_CurrentNode->value); //dangerous? <-- YES (but this lets PHP work on pages)
					}
				}
			break;
			
			case XMLReader::DOC_TYPE:
			
			break;
			
			case XMLReader::ENTITY_REF:
				if(count($gv_stopParseStack) == 0) {
					// Print if parsing
					print "[entity:" . $fo_CurrentNode->value . "]";
				}
			break;
		}
	}
} //function m_PullParse



?>