<?php
// From Symfony's Polyfills (MIT License for both)
// hash_equals - https://github.com/symfony/polyfill-php56/blob/master/Php56.php
// Binary class - https://github.com/symfony/polyfill/tree/master/src/Util
// Minor modifiations to integrate

/**
 * Binary safe version of string functions overloaded when MB_OVERLOAD_STRING is enabled.
 *
 * @author Nicolas Grekas <p@tchwork.com>
 *
 * @internal
 */
class BinaryOnFuncOverload
{
    public static function strlen($s)
    {
        return mb_strlen($s, '8bit');
    }

    public static function strpos($haystack, $needle, $offset = 0)
    {
        return mb_strpos($haystack, $needle, $offset, '8bit');
    }

    public static function strrpos($haystack, $needle, $offset = 0)
    {
        return mb_strrpos($haystack, $needle, $offset, '8bit');
    }

    public static function substr($string, $start, $length = 2147483647)
    {
        return mb_substr($string, $start, $length, '8bit');
    }

    public static function stripos($s, $needle, $offset = 0)
    {
        return mb_stripos($s, $needle, $offset, '8bit');
    }

    public static function stristr($s, $needle, $part = false)
    {
        return mb_stristr($s, $needle, $part, '8bit');
    }

    public static function strrchr($s, $needle, $part = false)
    {
        return mb_strrchr($s, $needle, $part, '8bit');
    }

    public static function strripos($s, $needle, $offset = 0)
    {
        return mb_strripos($s, $needle, $offset, '8bit');
    }

    public static function strstr($s, $needle, $part = false)
    {
        return mb_strstr($s, $needle, $part, '8bit');
    }
}

class BinaryNoFuncOverload
{
    public static function strlen($s)
    {
        return \strlen($s);
    }

    public static function strpos($haystack, $needle, $offset = 0)
    {
        return strpos($haystack, $needle, $offset);
    }

    public static function strrpos($haystack, $needle, $offset = 0)
    {
        return strrpos($haystack, $needle, $offset);
    }

    public static function substr($string, $start, $length = PHP_INT_MAX)
    {
        return substr($string, $start, $length);
    }

    public static function stripos($s, $needle, $offset = 0)
    {
        return stripos($s, $needle, $offset);
    }

    public static function stristr($s, $needle, $part = false)
    {
        return stristr($s, $needle, $part);
    }

    public static function strrchr($s, $needle, $part = false)
    {
        return strrchr($s, $needle, $part);
    }

    public static function strripos($s, $needle, $offset = 0)
    {
        return strripos($s, $needle, $offset);
    }

    public static function strstr($s, $needle, $part = false)
    {
        return strstr($s, $needle, $part);
    }
}

if (extension_loaded('mbstring')) {
    class Binary extends BinaryOnFuncOverload {}
} else {
    class Binary extends BinaryNoFuncOverload {}
}

if(!function_exists('hash_equals')) {
	function hash_equals($knownString, $userInput)
    {
        if (!\is_string($knownString)) {
            trigger_error('Expected known_string to be a string, '.gettype($knownString).' given', E_USER_WARNING);
            return false;
        }
        if (!\is_string($userInput)) {
            trigger_error('Expected user_input to be a string, '.gettype($userInput).' given', E_USER_WARNING);
            return false;
        }
        $knownLen = Binary::strlen($knownString);
        $userLen = Binary::strlen($userInput);
        if ($knownLen !== $userLen) {
            return false;
        }
        $result = 0;
        for ($i = 0; $i < $knownLen; ++$i) {
            $result |= \ord($knownString[$i]) ^ \ord($userInput[$i]);
        }
        return 0 === $result;
    }
}
?>