<?php


	class maml_persist extends prototype_tag {
		public $name='data';

		protected $doctext = <<<'DOCBLOCK'
<p><strong>Description:</strong> Creates a token and stores the contents of the tags in that token. 
Tokens persist until the browser session ends or times out, or until a &lt;maml:desist&gt; tag is
encountered.</p>
DOCBLOCK;
		
		public function begin() {
			$this->startCache();
		}
		
		public function end() {
			$this->finishCache();
			
			$_SESSION['tokens'][$this->name] = $this->tagCache;
		}
	}


	class maml_desist extends prototype_tag {
		public $name='data';

		protected $doctext = <<<'DOCBLOCK'
<p><strong>Description:</strong> destroys a token created by a &lt;maml:persist&gt; tag and
removes its data from memory.</p>
DOCBLOCK;
		
		public function begin() {
			$_SESSION['tokens'][$this->name] = '';
			unset($_SESSION['tokens'][$this->name]);
		}
		
	}

/* -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+----- */

	class maml_token extends prototype_tag {
		public $name;
		public $value;
		protected $doctext = <<<'DOCBLOCK'
<p><strong>Description:</strong> The &lt;maml:token&gt; tag allows you to set the value of a token for use later on the page. The token specified by the <strong>name</strong> attribute is set to the contents of the tag. You may also use the optional <strong>value</strong>
attribute which will override the contents of the tag and so should be used with he <em>empty tag</em> syntax for clarity.</p>
<pre class="maml"><span class="sc2">&lt;maml:token <span class="kw3">name</span><span class="sy0">=</span><span class="st0">&quot;Fred&quot;</span>&gt;</span>My Name is Fred.<span class="sc2">&lt;<span class="sy0">/</span>maml:token&gt;</span>
Token *<span class="sc2">&lt;span&gt;</span>[Fred]<span class="sc2">&lt;<span class="sy0">/</span>span&gt;</span>* is <span class="sc2">&lt;strong&gt;</span> *<span>[Fred]</span>* <span class="sc2">&lt;<span class="sy0">/</span>strong&gt;</span></pre>
<p>or</p>
<pre class="maml"><span class="sc2">&lt;maml:token <span class="kw3">name</span><span class="sy0">=</span><span class="st0">&quot;Fred&quot;</span> <span class="kw3">value</span><span class="sy0">=</span><span class="st0">&quot;My Name is Fred&quot;</span> <span class="sy0">/</span>&gt;</span>
Token *<span class="sc2">&lt;span&gt;</span>[Fred]<span class="sc2">&lt;<span class="sy0">/</span>span&gt;</span>* is <span class="sc2">&lt;strong&gt;</span> *<span>[Fred]</span>* <span class="sc2">&lt;<span class="sy0">/</span>strong&gt;</span></pre>
DOCBLOCK;
		
		public function begin() {
			$this->startCache();
		}
		
		public function end() {
			global $gv_Config, $gv_TokenVariables;
			$this->finishCache();
			
			if (!isset($this->value)) {
				$this->value = $this->tagCache;
			}
			$gv_Config[$this->name] = $this->value;
			$gv_TokenVariables[$this->name] = $this->value;
//			echo $this->name . ": " . $gv_TokenVariables[$this->name];
		}
	}




?>