/*
	filedrag.js - HTML5 File Drag & Drop demonstration
	Featured on SitePoint.com
	Developed by Craig Buckler (@craigbuckler) of OptimalWorks.net
	Modified for Molly by Gaurav Kumar, April 2016
*/
var t =  t || (function() {
	var _process = "";
	var _callback = "";
	
	// getElementById
	function $id(id) {
		return document.getElementById(id);
	};
	
	// output information
	function Output(msg) {
		var m = $id("messages");
		m.innerHTML = msg + m.innerHTML;
	};
	
	// file drag hover
	function FileDragHover(e) {
		e.stopPropagation();
		e.preventDefault();
		e.target.className = (e.type == "dragover" ? "hover" : "");
	};
	
	// file selection
	function FileSelectHandler(e) {	
		// cancel event and hover styling
		FileDragHover(e);
		
		// fetch FileList object
		var files = e.target.files || e.dataTransfer.files;
		
		//validateFileTypes: Validates file being uploaded to specified type 
		// Implemented by Gaurav
		if(validateFileTypes(files)){
			//trigger custom event 
			var fileSelectedEvent = new CustomEvent("onFileSelected", {
				detail: {
					message: {"data" : files, "fileAlreadyExist" : false, "fileDetail": null},
				},
				bubbles: true,
				cancelable: true
			});
			
			e.currentTarget.dispatchEvent(fileSelectedEvent);	
			
			//deliberately delay in order to get results from the callback to retrieve 'fileAlreadyExist' value
			window.setTimeout(function(){		
				if(! fileSelectedEvent.detail.message.fileAlreadyExist){
					// process all File objects*/			
					for (var i = 0, f; f = files[i]; i++) {		
						ParseFile(f);				
						UploadFile(f);
						
						//refresh page
						if(i == (files.length-1)){
							$( document ).ajaxStop(function() {
								location.reload();
							});	
						}
					}					
				}
				else{
					Output(
					"<p style='margin:auto; color:red;'>Error:Video "+ fileSelectedEvent.detail.message.fileDetail.name  + " already exists. Please delete the video and upload again</p>"
					);
					
				}
			}, files.length*1000);
			
		}
		else{	
			//display error if type does not match
		Output(
		"<p style='margin:auto; color:red;'>Error: File does not match the specified type</p>"
		);			
		}		
	};
	
	
	// output file information
	function ParseFile(file) {
		/*
			Output(
			"<p>File information: <strong>" + file.name +
			"</strong> type: <strong>" + file.type +
			"</strong> size: <strong>" + file.size +
			"</strong> bytes</p>"
			);
		*/		
		// display an image
		if (file.type.indexOf("image") == 0) {
			var reader = new FileReader();
			reader.onload = function(e) {
				Output(
				'<p><img src="' + e.target.result + '" /><br />' + file.name + '</p>'
				);
			}
			reader.readAsDataURL(file);
		}
		
		// display text
		if (file.type.indexOf("text") == 0) {
			var reader = new FileReader();
			reader.onload = function(e) {
				Output(
				"<p><strong>" + file.name + ":</strong></p><pre>" +
				e.target.result.replace(/</g, "&lt;").replace(/>/g, "&gt;") +
				"</pre>"
				);
			}
			reader.readAsText(file);
		}		
	};
	
	// upload JPEG files
	function UploadFile(file) {
		// following line is not necessary: prevents running on SitePoint servers
		if (location.host.indexOf("sitepointstatic") >= 0) return;	
		
		var xhr = new XMLHttpRequest();
		
		var statusSection = $id("statusContainer"),
		status = $id("uploadStatus"),		
		progressBar =  $id("uploadStatusBar");		
		statusSection.setAttribute("class", "upContainerShow");
		if (xhr.upload && file.size <= $id("MAX_FILE_SIZE").value){
			
			xhr.upload.addEventListener("progress", function(e){
				if(status && progressBar){
					var pc = parseInt(Math.round(e.loaded / e.total * 100)); 	
					status.innerHTML = "complete: " + pc + "%"; 
					progressBar.value =  pc ;		
				}								
			},false);
			
			
			xhr.addEventListener("load", function(e){				
				if(xhr.responseText.length > 0){
					var data = JSON.parse(xhr.responseText);
					if(data[0].uploaded){
						// file uploaded successfully	
						//trigger custom event 
						var fileUploaded = new CustomEvent("onFileUploaded", {
							detail: {
								message : {"data" : file, "path": basePath, "filePostUploadProcessingError" : false, "fileDetail": null},
							},
							bubbles: true,
							cancelable: true
						});
						
						statusSection.dispatchEvent(fileUploaded);
						
						setTimeout(function (){
							if(fileUploaded.detail.message.filePostUploadProcessingError){
								Output(
								"<p style='margin:auto; color:red;'>Error: Error occurred while saving uploaded file information in database</p>"
								);	
							}							
						}, 2000);
						
						// callback script 
						if(_callback.length > 0)
						appendCallbackScript(_callback);
						
						// call to file to process
						if(_process.length > 0)
						executeProcess(_process);																			
					}
				}
			}, false);
			
			xhr.addEventListener("error", function(evt){
				status.innerHTML = "An error occurred while transferring the file";				
			}, false);
			
			
			// file received/failed
			xhr.onreadystatechange = function(e) {
				if (xhr.readyState == 4) {									
					if (xhr.status == 200) {						
						status.innerHTML = file.name + " upload complete.";							
					}
					else{						
						status.innerHTML = file.name + " upload failed.";						
					}										
				}
			};
			
			xhr.ontimeout = function(e){
				status.innerHTML = "Error: Request timed out";						
			}
			
			// start upload
			xhr.open("POST", $id("upload").action, true);
			xhr.setRequestHeader("X_FILENAME", file.name);
//			xhr.send(file);		
			var formData = new FormData();
			formData.append("file", file);
			xhr.send(formData);	
		}
		
	};
	
	// initialize
	function Init() {
		
		var fileselect = $id("fileselect"),
		filedrag = $id("filedrag"),
		submitbutton = $id("submitbutton");
		
		// file select
		fileselect.addEventListener("change", FileSelectHandler, false);
		
		// is XHR2 available?
		var xhr = new XMLHttpRequest();
		if (xhr.upload) {
			
			// file drop
			filedrag.addEventListener("dragover", FileDragHover, false);
			filedrag.addEventListener("dragleave", FileDragHover, false);
			filedrag.addEventListener("drop", FileSelectHandler, false);
			filedrag.style.display = "block";
			
			// remove submit button
			submitbutton.style.display = "none";
		}
		
	};
	
	function appendCallbackScript(src){
		var head= document.getElementsByTagName('head')[0];
		var script= document.createElement('script');
		script.type= 'text/javascript';
		script.src= src;
		head.appendChild(script);		
	};
	
	function executeProcess(url){
		var msg = "";
		var oXHR = new XMLHttpRequest();		
		oXHR.open("GET", url, true);		
		oXHR.onreadystatechange = function (oEvent) {  
			if (oXHR.readyState === 4) {  
				if (oXHR.status === 200) {  
					msg = "Upload post processing successfully completed";
					} else { 
					msg = "Upload post processing error : " +oXHR.statusText;				
				} 				
				Output(
				"<p>" + msg + "</p>"
				);
			}  
		}; 
		
		oXHR.send(null);			
	}
	
	function validateFileTypes(files){
		var fCount = files.length;
		var acceptTypes = $id("fileselect").accept;
		if(acceptTypes && fCount > 0 ){
			
			acceptTypes = acceptTypes.toLowerCase().trim();	
			
			for(var i = 0; i < fCount; i++) {
				var f = files[i]; 
				var fType;
				
				if(f.type.indexOf('/')){
					fType = f.type.split("/");
					
					if(acceptTypes.indexOf(fType[0]) === -1){						
						if(acceptTypes.indexOf(fType[1]) === -1){
							return false;
						}
					}
					
				}
				
			}
			
		}
		return true;
	};
	
	return {
		uploadInitializeParam : function(param1, param2){
			// initialize variables			
			_callback = param1;	
			_process = param2;
			if(_callback.length > 0)
			appendCallbackScript(_callback);
			if (window.File && window.FileList && window.FileReader) {
				Init();							
			}			
		}
	};
	
})();								